t_name = "PREPARED_T_%d"
log_file = 'log.txt'
import psycopg2

class logger:
    def __init__(self, conns):
        self.conns = conns
        self.tid = 0


    def log(self,tid, type):
        f = open(log_file,'a')
        f.write("%s %d\n"%(type, tid))
        f.close()


    def start_coord_txn(self):
        cur_tid = self.tid
        self.log(cur_tid, 'START')
        self.tid = self.tid + 1
        return cur_tid

    def start_worker_txn(self, cursor, tid):
        cursor.execute("BEGIN TRANSACTION")

    def prepare(self, cursor, tid):
        try:
            cursor.execute ("prepare transaction '%s'"%(t_name%(tid)))
            return True
        except psycopg2.DatabaseError as error:
            print(error)
            return False


    def commit(self, cursor, tid):
        try:
            cursor.execute ("commit prepared '%s'"%(t_name%(tid)))
        except psycopg2.DatabaseError as error:
            print(error)

    def abort(self, cursor, tid):
        try:
            cmd = "rollback prepared '%s'"%(t_name%(tid))
            print('issuing %s'%(cmd))
            cursor.execute (cmd)
        except psycopg2.DatabaseError as error:
            print(error)


    def log_lines(self):
        ret = []
        try:
            with open(log_file,'r') as f:
                lines = f.readlines()
                for l in lines:
                    cmds = l.split(" ")
                    tid = int(cmds[1])
                    ret.append((tid,cmds[0]))
        except Exception as e:
            print(e)
            print("no log file, continuing")
        return ret

    def get_workers(self):
        workers = []
        for conn in self.conns:
            worker = conn.cursor()
            workers.append(worker)
        return workers

    def recover(self):
        to_abort = []
        to_commit = []
        max_tid = 0
        for (tid,cmd) in self.log_lines():
            if cmd == 'START':
                to_abort.append(tid)
                max_tid = max(self.tid, tid)
            if cmd == 'COMMIT':
                to_abort.remove(tid)
                to_commit.append(tid)
            #txns logged as 'ABORT' will implicitly abort
            if cmd == 'DONE':
                if tid in to_abort:
                    to_abort.remove(tid)
                if tid in to_commit:
                    to_commit.remove(tid)
        if (len(to_abort) > 0 or len(to_commit) > 0):
            workers = self.get_workers()
            for txn in to_abort:
                print("aborting tid %d"%(txn))
                for worker in workers:
                    self.abort(worker,txn)
                self.log(txn,'DONE') # don't do this again
            for txn in to_commit:
                print("committing tid %d"%(txn))
                for worker in workers:
                    self.commit(worker,txn)
                self.log(txn,'DONE') # don't do this again

            for worker in workers:
                worker.close()

        self.tid = max_tid + 1


